#!/usr/bin/perl

# Expects a comment in the POST parameters

use strict;
use warnings;
use utf8;
use open qw/:std :encoding(utf-8)/;

use CGI::Simple;
use File::Slurper qw/read_text/;
use Data::Dumper;
use POSIX 'strftime';
use File::Temp qw/tempfile/;

$CGI::Simple::POST_MAX = 8192; # 8 Kib
$CGI::Simple::DISABLE_UPLOADS = 1;
$CGI::Simple::PARAM_UTF8 = 1;
#$CGI::Simple::DEBUG = 1;

my $q = CGI::Simple->new;

my @header = qw/-charset utf-8/;
my $dir = "_comments";
my $suffix = ".comment";
my $queue_limit = 50;

unless ( -t STDIN || ( $ENV{REQUEST_METHOD} // '' ) eq 'POST' )
{
    my $status = '405 Method Not Allowed';
    print $q->header( @header, -status => $status );
    print_errorpage( $status );
    exit 0;
}

# When no 'article' parameter has been given, try to extract the
# article id (slug) from the URI
my $article_slug = $ENV{REQUEST_URI} =~ s,/comment(\?.*)?$,,rn;
my %c = (
    article  => scalar $q->param( 'article' ) // $article_slug,
    author   => scalar $q->param( 'author' ),
    body     => scalar $q->param( 'body' ),
    password => scalar $q->param( 'recog' ) // '',
    parent   => scalar $q->param( 'replyto' ) // '',
);

unless ( $c{article} && $c{article} =~ m,^[-/a-z0-9]+$,
      && $c{author} && 0 < length $c{author} < 64 && no_weird_chars( $c{author} )
      && $c{body} && 0 < length $c{body} < 8000 && no_weird_chars( $c{body} )
      && ( !$c{parent} || $c{parent} =~ /^[0-9]{14}\w{4}$/ ) )
{
    my $status = '400 Bad Request';
    print $q->header( @header, -status => $status );
    print_errorpage( $status );
    exit 0;
}

sanitise( $c{author} );
sanitise( $c{password} );

my $passed = $c{password} eq 'strasnetajneheslo'; # TODO

# Deny if the queue is full, unless the password is correct
my @ls = glob( "$dir/*$suffix" );
if ( !$passed && @ls >= $queue_limit )
{
    my $status = '503 Service unavailable';
    print $q->header( @header, -status => $status );
    print_errorpage( $status );
    exit 0;
}

$c{parent} ||= 'root'; # top-level comments

$c{body} =~ s/\v/\n/sg;
$c{body} =~ s/^\n*//s;
$c{body} =~ s/\n*$//s;

my @now = localtime;
my $date = strftime "%F %T %Z", @now;
my $slug = strftime "%Y%m%d%H%M%S", @now;

my ($fh, $filename) = tempfile( "${slug}XXXX", DIR => $dir, SUFFIX => $suffix );
my $commentid = $filename =~ s/^\Q$dir\E\/(.+)\Q$suffix\E$/$1/r;

unless ( -t STDIN ) {
    my $gid = getgrgid "adamat";
    chown -1, $gid, $filename;
    chmod 0440, $filename;
}

my $comment = <<EOF;
Article: $c{article}
Author:  $c{author}
Date:    $date
Pass:    @{[ $passed ? 'yes' : 'no' ]}\t$c{password}
Stamp:   @{[time]}
Id:      $commentid
Parent:  $c{parent}

$c{body}
EOF

print $fh $comment;
close $fh;

print $q->header(
    -status => '303 See Other',
    -location => '/commented.html',
);
exit 0;


sub print_errorpage {
    my $status = shift;
    my ($code) = $status =~ /^(\d+)/;
    my $file = "/var/www/errorpages/$code.html";
    if ( -r $file ) {
        print read_text( $file );
    }
    else {
        print "Status $status\n";
    }
}

sub no_weird_chars {
    $_ = shift;
    return /^[\p{Print}\n]*$/s;
}

sub sanitise {
    for ( shift ) {
        s/\s+/ /gs;
        s/^ +//;
        s/ +$//;
    }
}

# vim: sts=4 et ts=8 sw=4
